/*******************************************************************************


    ALERTA_falante_farol.C

    Alertador com mensagens previamente gravadas em EEprom 27c512:
    
    1) Para avisar que dever ligar os faris baixo quando ligar a chave de
    contato (Brasil).
    
    2) Para indicar esquecimento de lanternas ligadas ao sair
    do veculo

    Modo PWM - utiliza o PWM interno do pic a 19,53 khz c/ arquivos a 6,51 khz.

    Autor: Claudio Lrios

    Data: 11/07/2016

    Microcontrolador: PIC 16f628a

    C/ EEprom 24c512 para guardar a mensagem:"Desligue as lanternas" e "Ligue os
    faris baixos se for transitar na rodovia".

    Pwm com freq. Amostragem= 19,53khz
    No entanto, os arquivos na EEprom  foram feitos com freq. amostragem pela
    tera parte (6,51 khz). Isto reduz o tamanho da memria. Para obter o som
    correto cada byte de som  convertido 3 vezes seguidos. Isto elimina o
    apito caracteristico na reproduo do som quando se usa arquivos com freq.
    amostragem abaixo de 15khz.


    Este arquivo  parte integrante do blog 'LARIOS.TECNOLOGIA.WS'
    Proposta didtica apenas, sendo testada somente em placa de protoboard.

******************************************************************************/


#include <16F628A.h>
#use delay(clock=20000000)//USE XTAL DE 20MHZ
#fuses NOWDT,HS, NOPUT, NOPROTECT, BROWNOUT, NOMCLR, NOLVP, NOCPD
#use fast_io(a)
#use fast_io(b)
//#ZERO_RAM
#rom 0x2100 = {0,0,0}

#byte trisa= 0x85
#byte trisb= 0x86
#byte porta= 0x05
#byte portb= 0x06
#byte pr2=   0x92
#byte ccp1l= 0x15
#byte option_reg = 0x81

#bit    scl          = 0x05.2  //pino 1 ra2
#bit    sda          = 0x05.3  //pino 2 ra3
#bit    in_ign       = 0x06.1  //pino 7 rb1
#bit    in_porta     = 0x06.2  //pino 8 rb2
#bit    in_farol     = 0x06.4  //pino 10 rb4


#bit    flagt2       = 0x0C.1  //flag do timer 2
#bit    som          = 0x06.3  //pino 9 saida de som
#bit    tris_som     = 0x86.3  //sentido
#bit    tris_sda     = 0x85.3  // sentido
#bit    in_lant      = 0x86.0  //pino 6 rb0

int8 const  tcl  =  1;


#define  porta_neg

#define tempo_init 5000
/*==============================================================================
       SUB-ROTINAS   I2C
==============================================================================*/

/*==============================================================================
                            START
==============================================================================*/
void start_i2c(){
  tris_sda=1; //alta impedancia
  scl=1; //
  delay_us(tcl);
  sda=0;
  tris_sda=0; //saida zerada
  delay_us(tcl);
  scl=0;  //zera scl
  delay_us(tcl);
  tris_sda=1; //alta impedancia
}
/*==============================================================================
                            STOP
==============================================================================*/


void stop_i2c(){
  sda=0;
  tris_sda=0;
  scl=0;
  delay_us(tcl);
  scl=1;
  delay_us(tcl);
  tris_sda=1;//sda =1 (por alta impedancia + resistor pullup)
}
/*==============================================================================
                            ACKNOLEDGE
==============================================================================*/

void ack(){
   sda=0;
   tris_sda=0;
   scl=1;
   delay_us(tcl);
   scl=0;
   delay_us(tcl);
   tris_sda=1;//sda =1 (por alta impedancia + resistor pullup)

}
/*==============================================================================
                          NO_ACKNOLEDGE
==============================================================================*/
void no_ack(){

   tris_sda=1; //sda =1 (por alta impedancia + resistor pullup)
   scl=1;
   delay_us(tcl);
   scl=0;
   delay_us(tcl);
  }
/*==============================================================================
                      ENVIA BYTE PARA EEPROM 24C256/512
==============================================================================*/
void send_i2c(int b){
int a;
  tris_sda=0;
  for(a=0;a<8;a++){
  if(b&0x80){ sda=1;} else {sda=0;}
  scl=1;
  delay_us(tcl);
  scl=0;
  b=b<<1;
  }

}

/*==============================================================================
     ROTINA DE REPRODUO DE SOM GRAVADO NA MEMRIA EEPROM EXTERNA
==============================================================================*/

void reproduzir(int16 init,int16 end){
 int8 buf,d;


   no_ack();
   stop_i2c();//para uma reproduo anterior interrompida

   start_i2c(); //inicia condio de start
   send_i2c(0xa0);//device write
   ack();
   send_i2c(make8(init,1));// endereo alto a ser reproduzido
   ack();
   send_i2c(make8(init,0));//endereo baixo a ser reproduzido
   ack();

   start_i2c(); //inicia condio de start
   send_i2c(0xa1);//device read
   ack();
   tris_sda=1; //entrada

  tris_som=0;//port de som como sada
  for(;;){// reproduz at encontrar endereo final
       for(d=0;d<8;d++){
         shift_left(&buf,1,sda); //recebe o mais significativo primeiro
         scl=1;
         delay_us(tcl);//fornece um clock
         scl=0;
      }
      ack();//responde
      if(++init==end) {tris_som=1; break;} //para no fim do arquivo

      while(!flagt2) ;//aguarda 'pwm' finalizar processo
      ccp1l=buf;//carga do valor do pwm
      flagt2=0;//zera flag tmr2


      while(!flagt2) ;//aguarda 'pwm' finalizar processo
      ccp1l=buf;//repete carga do valor do pwm para usar arquivo de 6510hz
      flagt2=0;//zera flag tmr2

    while(!flagt2) ;//aguarda 'pwm' finalizar processo
      ccp1l=buf;//repete carga do valor do pwm para usar arquivo de 6510hz
      flagt2=0;//zera flag tmr2



  }
      tris_som=1;
}

/*==============================================================================

                      ROTINA MAIN

==============================================================================*/

void main() {

   setup_counters(RTCC_INTERNAL,RTCC_DIV_1);
   setup_timer_1(T1_DISABLED);
   setup_timer_2 ( T2_DIV_BY_1, 0, 1);
   setup_comparator(NC_NC_NC_NC);
   setup_ccp1(CCP_PWM);
   setup_vref(FALSE);
   trisa=0b11111011;
   trisb=0b11110111;
   pr2=0xff;//periodo max
   option_reg&=0x7f; //pullup ligado no portb
   stop_i2c();
   delay_ms(2000);//delay

/*==============================================================================

                      LOOP PRINCIPAL

==============================================================================*/
  for(;;){
/*******************************************************************************
REPRODUZ ARQUIVO GRAVADO PREVIAMENTE NA EEPROM 24C512
*******************************************************************************/
//condio para o aviso: lanterna ligada, porta aberta e ignio desligada
   static unsigned int1 flag_ign=0;
     static unsigned int1 flag_lant=0;

      #ifdef  porta_neg
      if(in_ign && in_porta && !flag_lant && in_lant) // se interruptor da porta for ligado ao negativo
      #else
      if(in_ign && !in_porta && !flag_lant && in_lant)// se interruptor da porta for ligado ao positivo
      #endif

      {
       reproduzir(0xb180,0xffff);//mensagem:'dum...dum..dum..Desligue as lanternas'

      flag_lant=1;
      }




      #ifdef porta_neg
      if(!in_porta)
      #else
      if(in_porta)
      #endif
      {
        flag_lant=0;
      }


      #ifdef porta_neg
        if(in_farol && !in_ign && !flag_ign && !in_porta)
      #else
       if(in_farol && !in_ign && !flag_ign && in_porta)
      #endif
       {

    
         delay_ms(tempo_init);
         reproduzir(0,0xb17f);//mensagem: 'ligue os faris...'
         flag_ign=1;


      }

      if(in_ign && in_farol)flag_ign=0;//se chave de contato for desligada


   }//for(;;)


}//main
